------------------------------------------------------

--               PORTE 3 ETATS

------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

 

entity tristate is

  port( oe : in std_logic;

        data_in  : in std_logic_vector( 31 downto 0);

        data_out : inout std_logic_vector(31 downto 0));

end tristate;

 

architecture arch_tristate of tristate is

begin

 

 data_out <= data_in when oe='1' else ( others => 'Z' );

 

end arch_tristate;  

      

------------------------------------------------------

--               MULTIPLEXEUR MUX A 2 VOIES

------------------------------------------------------

library IEEE;

use IEEE.std_logic_1164.all;

use IEEE.numeric_std.all;



entity mux_2 is 

  port( sel : in std_logic;

        e_0, e_1 : in std_logic_vector(31 downto 0);

        s : out std_logic_vector( 31 downto 0));

end mux_2;



architecture arch_mux_2 of mux_2 is

  begin

    

s <= e_0 when sel='0' else e_1;

    

end arch_mux_2;   



------------------------------------------------------

--               MULTIPLEXEUR MUX  5 VOIES

------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity mux_5 is 
  port( sel : in std_logic_vector( 3 downto 0 );
        e_0, e_1, e_2, e_3, e_4 : in std_logic_vector(31 downto 0);
        s : out std_logic_vector( 31 downto 0));
end mux_5;

architecture arch_mux_5 of mux_5 is
  begin
  
s <= 	e_0 when sel="0000"
	else e_1 when sel="0001"
	else e_2 when sel="0010"
	else e_3 when sel="0011"
	else e_4 when sel="0100"
	else ( others => '0' ) ;  

end arch_mux_5;    



------------------------------------------------------

--          UNITE ARITHMETIQUE ET LOGIQUE

------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.up_pack.all;

entity alu is 

  port ( A, B : in std_logic_vector(31 downto 0);
         alufs : in ALU_FCTS;
	n, z , v, c : out std_logic;

          S : out std_logic_vector( 31 downto 0));

end alu;



architecture arch_alu of alu is

signal copy_s : unsigned( 32 downto 0);

begin

copy_s <=  	unsigned('0'&A) + unsigned('0'&B) when alufs = ALU_ADD
else  		unsigned('0'&A) - unsigned('0'&B) when alufs = ALU_SUB
else  		unsigned('0'&A) and unsigned('0'&B) when alufs = ALU_AND
else  		unsigned('0'&A) or unsigned('0'&B) when alufs = ALU_OR
else  		unsigned('0'&A) xor unsigned('0'&B) when alufs = ALU_XOR
else  (others => '0');

n <= copy_s(31);
z <= '1' when copy_s = "0000000000000000000000000000000" else '0';
v <= '1' when (( A(31)='0') and ( B(31)='0') and ( copy_s(31)='1')) or
		(( A(31)='1') and ( B(31)='1') and ( copy_s(31)='0')) else '0' ;
c <= copy_s(32);
S <= B when alufs = ALU_B else std_logic_vector(copy_s(31 downto 0));
end arch_alu;          



------------------------------------------------------

--               REGISTRE 32 BITS

------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity registre is
  port( clk, raz, load : in std_logic;
        data_in : in std_logic_vector(31 downto 0);
        data_out : out std_logic_vector(31 downto 0));    
end registre;

architecture arch_reg of registre is

  begin   
   process(clk,raz)
        begin
      
        if rising_edge(clk) then

	    if raz='1' then data_out <= (others => '0');

            elsif load='1' then data_out <= data_in;

            end if;

        end if;

    end process;

  

end arch_reg;

------------------------------------------------------

--               REGISTRE 4 BITS

------------------------------------------------------

library IEEE;

use IEEE.std_logic_1164.all;

use IEEE.numeric_std.all;



entity registre_4b is

  port( clk, raz, load : in std_logic;

        data_in : in std_logic_vector(3 downto 0);

        data_out : out std_logic_vector(3 downto 0));

        

end registre_4b;



architecture arch_reg_4b of registre_4b is



  begin   



   process(clk,raz)

        begin

        

        if rising_edge(clk) then

	    if raz='1' then data_out <= (others => '0');

            elsif load='1' then data_out <= data_in;

            end if;

        end if;

    end process;

  

end arch_reg_4b;



------------------------------------------------------

--               REGISTRE IR (Instruction Register)

------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.up_pack.all; 

entity ir_reg is
  port( clk, raz, load : in std_logic;
        data_in : in std_logic_vector(31 downto 0);
        opcode : out std_logic_vector(7 downto 0);
		sel_dest : out std_logic_vector(3 downto 0);
		sel_op1 : out std_logic_vector(3 downto 0);
		sel_op2 : out std_logic_vector(3 downto 0);
		index : out std_logic_vector(7 downto 0));        
end ir_reg; 



architecture arch_ir_reg of ir_reg is
 signal interne :  std_logic_vector(31 downto 0);
begin  
   process(clk)
        begin
        if rising_edge(clk) then
	    if raz='1' then interne <= (others => '0');
            elsif load='1' then interne <= data_in;
            end if;
        end if;
    end process;
	opcode <= interne(31 downto 24);
	sel_dest <= interne ( 19 downto 16);
	sel_op1 <= interne ( 15 downto 12);
	sel_op2 <= interne ( 11 downto 8);
	index <= interne( 7 downto 0 );
end arch_ir_reg; 



library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all; -- types signed / unsigned (calcul)

entity add is
port(   A     :   in std_logic_vector(31 downto 0);
  	    B     :   in std_logic_vector(7 downto 0);
        result  :   out std_logic_vector(31 downto 0));
end add;

architecture arch_add of add is
 signal extension : std_logic_vector(23 downto 0);
begin
    extension <= ( others => B(7));
    result <=    std_logic_vector(unsigned(A)+unsigned(extension&B));
end arch_add;


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all; -- types signed / unsigned (calcul)

 

entity increment is
port(       n     :   in std_logic_vector(31 downto 0);
            n_plus :   out std_logic_vector(31 downto 0));
end increment;

 

architecture arch_increment of increment is
begin
   n_plus <=    std_logic_vector(unsigned(n)+4);
end arch_increment;


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all; -- types signed / unsigned (calcul)

 
entity decodeur is

port( E : in std_logic_vector(3 downto 0 );
	enable :  in std_logic;
	s0, s1, s2, s3, s4 : out std_logic);
end decodeur;



architecture arch_decodeur of decodeur is

begin

	s0 <= '1' when ( E="0000")  and ( enable = '1') else '0';
	s1 <= '1' when ( E="0001")  and ( enable = '1') else '0';	
	s2 <= '1' when ( E="0010")   and ( enable = '1') else '0';	
	s3 <= '1' when ( E="0011")  and ( enable = '1') else '0';	
	s4 <= '1' when ( E="0100")  and ( enable = '1') else '0';


end arch_decodeur;

------------------------------------------------------

--          SEQUENCEUR (deja fourni,  analyser)

------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.up_pack.all;  

entity fsm is
  port( clk, reset : in std_logic;
        opcode_in : in std_logic_vector( 7 downto 0);
        sel_dest : in std_logic_vector( 3 downto 0);
        sel_op2 : in std_logic_vector( 3 downto 0);
		nzvc : in std_logic_vector( 3 downto 0);
        raz  : out std_logic;
		memrq, rnw : out std_logic; 
		alufs : out ALU_FCTS;
		data_bus_oe : out std_logic;
		sel_db_dest, sel_pc_res : out std_logic;
		sel_inc_pc : out std_logic;  
		ld_pc, ld_ir, ld_sr : out std_logic;
		ld_r0, ld_r1, ld_r2, ld_r3 : out std_logic);


        

        

end fsm;



architecture arch_fsm of fsm is
  type me_states is (INIT, FETCH, EXECUTE, STOP);
  signal  etat_cr, etat_sv : me_states ;
  signal opcode : OPCODE;
  signal en_dest : std_logic;
begin

opcode <= OP_LDR    when opcode_in="00000000" else
          OP_STR    when opcode_in="00000001" else
		  OP_MOV    when opcode_in="00000010" else
          OP_ADD    when opcode_in="00000011" else
          OP_SUB    when opcode_in="00000100" else
		  OP_AND    when opcode_in="00000101" else
          OP_OR     when opcode_in="00000110" else
          OP_XOR    when opcode_in="00000111" else
          OP_B	    when opcode_in="00001000" else
          OP_BHI    when opcode_in="00001001" else
          OP_BNE    when opcode_in="00001010" else
          OP_STP    when opcode_in="00001111" else
          OP_UNKNOWN;

ld_r0 <= '1' when ( sel_dest="0000" ) and (en_dest = '1') else '0';
ld_r1 <= '1' when ( sel_dest="0001" ) and (en_dest = '1') else '0';
ld_r2 <= '1' when ( sel_dest="0010" ) and (en_dest = '1') else '0';
ld_r3 <= '1' when ( sel_dest="0011" ) and (en_dest = '1') else '0';
ld_pc <= '1' when (( sel_dest="0100" ) and (en_dest = '1')) or (etat_cr = FETCH ) else '0';

process( clk , reset)
    begin
      if reset='1' then etat_cr <= INIT;
      elsif rising_edge(clk) then etat_cr <= etat_sv; 
      end if;     
end process;   
    

process(etat_cr, opcode, nzvc)
    begin
etat_sv <= etat_cr; raz <= '0'; 
ld_ir <= '0'; 
alufs <= ALU_B;
memrq <= '1'; rnw <='1';
sel_pc_res <= '0';

sel_inc_pc<= '0';
ld_sr <='0';
sel_db_dest <='0';
data_bus_oe <='0';
en_dest <='0';

		case etat_cr is
			when INIT =>
				etat_sv <= FETCH;
				raz <= '1';
			when FETCH =>
				etat_sv <= EXECUTE;
				ld_ir <= '1';				-- load IR with data on the data bus
				
				sel_inc_pc <= '1';			-- load PC with PC+1
			when EXECUTE =>
				case opcode is
					when OP_LDR =>			 
						sel_db_dest <= '0';							
 						sel_pc_res <= '1';
						alufs <= ALU_B;		-- op = B	-- load data in accumulator
						etat_sv <= FETCH;	
						en_dest <= '1';				
					when OP_STR =>			
						sel_db_dest <= '0';						
 						sel_pc_res <= '1';
						alufs <= ALU_B;		-- op = B	-- load data in accumulator
						etat_sv <= FETCH;
						data_bus_oe <= '1';
						rnw <='0';
						en_dest <= '1';		
					when OP_ADD =>			
						sel_db_dest <= '1';	-- IR[mem] on addr bus						
						alufs <= ALU_ADD;		-- op = B	-- load data in accumulator
						etat_sv <= FETCH;
						ld_sr <= '1';		
						en_dest <= '1';							
					when OP_MOV =>			
						sel_db_dest <= '1';						
						alufs <= ALU_B;		
						etat_sv <= FETCH;
						ld_sr <= '1';		
						en_dest <= '1';												
					when OP_SUB =>			
						sel_db_dest <= '1';						
						alufs <= ALU_SUB;		
						etat_sv <= FETCH;
						ld_sr <= '1';		
						en_dest <= '1';							
					when OP_BNE =>			 
						sel_db_dest <= '1';							
 						sel_pc_res <= '0';
						alufs <= ALU_B;		
						etat_sv <= FETCH;	
						if NZVC(2) = '0' then en_dest <= '1'; end if;											
					when OP_B=>			 
						sel_db_dest <= '1';						
 						sel_pc_res <= '0';
						alufs <= ALU_B;		
						etat_sv <= FETCH;	
						en_dest <= '1';	
					when others =>
						etat_sv <= STOP;
				end case;

				

			when STOP => etat_sv <= STOP;

		end case;	  

 

end process;



end arch_fsm;








