------------------------------------------------------


--       INSTANCIATION : DESCRIPTION DE mu0


------------------------------------------------------

library IEEE;

use IEEE.std_logic_1164.all;

use IEEE.numeric_std.all;

use work.up_pack.all; 

use work.all;


entity mu0 is

  port( 

    reset, clk : in std_logic;

    data_bus : inout std_logic_vector(31 downto 0);

    addr_bus : out   std_logic_vector(31 downto 0);

    mem_rq  :  out   std_logic;

    rnw     :  out   std_logic);

  end mu0;


  


architecture arch_mu0 of mu0 is


 signal data_bus_oe : std_logic	;

 signal to_data_bus : std_logic_vector( 31 downto 0);		                    -- raz for registers

 signal pc_out : std_logic_vector( 31 downto 0);	

 signal res_alu : std_logic_vector( 31 downto 0);
signal n_plus : std_logic_vector( 31 downto 0);

 signal sel_pc_res : std_logic	;	



 signal sel_inc_pc : std_logic;

 signal sel_db_dest : std_logic;



 signal ld_pc : std_logic;

 signal raz : std_logic;

 signal n,z,v,c : std_logic;

 signal alufs : ALU_FCTS;

 signal op1, op2 : std_logic_vector( 31 downto 0);

 signal mux_op2_out : std_logic_vector( 31 downto 0);

 signal concat : std_logic_vector( 31 downto 0);

 signal pc_in : std_logic_vector( 31 downto 0);

signal index : std_logic_vector( 7 downto 0);

 signal pc_plus_un : std_logic_vector( 31 downto 0);

 signal reg_in, r0_out, r1_out, r2_out, r3_out: std_logic_vector( 31 downto 0);

 signal ld_r0, ld_r1, ld_r2, ld_r3, ld_sr, ld_ir : std_logic; 

 signal sel_op1, sel_op2, sel_dest : std_logic_vector( 3 downto 0);

 signal nzvc, sr_out : std_logic_vector( 3 downto 0);

signal opcode : std_logic_vector(7 downto 0);

signal en_dest_fsm : std_logic;

begin

porte_3e : entity tristate port map ( 

	oe => data_bus_oe,

        data_in  => to_data_bus,

        data_out => data_bus);


multiplexeur_2 : entity mux_2 port map ( 

  	sel => sel_pc_res,

        e_0 => pc_out,

	e_1 => res_alu,

        s   => addr_bus

);



multiplex_2_regin : entity mux_2 port map ( 

  	sel => sel_db_dest,

        e_0 => data_bus,

	e_1 => res_alu,

        s   => reg_in

);

multiplex_2_pc : entity mux_2 port map ( 

  	sel => sel_inc_pc,

        e_0 => reg_in,

	e_1 => n_plus,

        s   => pc_in

);




alu0 : entity alu port map ( 

  	A => op1,

	B => op2,

        alufs => alufs,

	n => n,

	z => z,

	v => v,

	c => c,

        S => res_alu

);



add0 : entity add port map (

	A => mux_op2_out,

	B  => index,   

        result => op2

);


inc0 : entity increment port map (

    n => pc_out,

    n_plus => n_plus

);




reg0 : entity registre port map (

  	clk  => clk, 

	raz  => raz,

	load  => ld_r0,

        data_in  => reg_in,

        data_out  => r0_out

);





reg1 : entity registre port map (

  	clk  => clk, 

	raz  => raz,

	load  => ld_r1,

        data_in  => reg_in,

        data_out  => r1_out

);    


reg2 : entity registre port map (

  	clk  => clk, 

	raz  => raz,

	load  => ld_r2,

        data_in  => reg_in,

        data_out  => r2_out

);



reg3 : entity registre port map (

  	clk  => clk, 

	raz  => raz,

	load  => ld_r3,

        data_in  => reg_in,

        data_out  => r3_out

);


pc : entity registre port map (

  	clk  => clk, 

	raz  => raz,

	load  => ld_pc,

        data_in  => pc_in,

        data_out  => pc_out

);


mux_op1 : entity mux_5 port map ( 

	sel => sel_op1,

        e_0 => r0_out, 

	e_1 => r1_out, 

	e_2 => r2_out, 

	e_3 => r3_out, 

	e_4 => pc_out ,

        s => op1

);


mux_op2 : entity mux_5 port map ( 

	sel => sel_op2,

        e_0 => r0_out, 

	e_1 => r1_out, 

	e_2 => r2_out, 

	e_3 => r3_out, 

	e_4 => pc_out ,

        s => mux_op2_out

);





mux_dest : entity mux_5 port map ( 

	sel => sel_dest,

        e_0 => r0_out, 

	e_1 => r1_out, 

	e_2 => r2_out, 

	e_3 => r3_out, 

	e_4 => pc_out ,

        s => to_data_bus


);


nzvc <= n&z&v&c;




sr : entity registre_4b port map (

	clk  => clk, 

	raz  => raz,

	load  => ld_sr,

        data_in  => nzvc,

        data_out  => sr_out

);





ir : entity ir_reg port map (

	clk => clk,

	raz => raz,

	load => ld_ir,

        data_in => data_bus,

        opcode => opcode,

	sel_dest => sel_dest,

	sel_op1 => sel_op1,

	sel_op2 => sel_op2,

	index => index

);





me : entity fsm port map (

	clk => clk, 

	reset => reset,

        opcode_in => opcode,
    sel_dest => sel_dest,
        sel_op2 => sel_op2,

	nzvc => sr_out,

        raz  => raz,

	memrq => mem_rq,

	rnw => rnw, 

	alufs => alufs, 

	data_bus_oe => data_bus_oe,

	sel_db_dest => sel_db_dest, 

	sel_pc_res => sel_pc_res,

	

	sel_inc_pc =>  sel_inc_pc, 

	ld_pc => ld_pc, 

	ld_ir => ld_ir, 

	ld_sr => ld_sr,  

ld_r0 => ld_r0,  
ld_r1 => ld_r1,

ld_r2 => ld_r2,    
ld_r3 => ld_r3

);       





end arch_mu0 ;
